;-------------------------------------------------------------------------------
;	Copyright (c) 2003-2004 by Matt Brunk
;	web:    http://www.smartcache.net/
;	e-mail: info@smartcache.net
;
;	The contents of this file are subject to the Artistic License (the "License").
;	You may not use this file except in compliance with the License. 
;	You may obtain a copy of the License at:
;	http://www.opensource.org/licenses/artistic-license.html
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;	This source code implements an interface between an RS-232 serial port and 
;	an ISO 7816 smart card, and is targeted for the Atmel AT90S2313 
;	microcontroller.  The frequency of the clock signal applied to the Atmel 
;	microcontroller is assumed to be 9.216 MHz.  As a result, the 10 MHz 
;	version of the microcontroller must be used (e.g., AT90S2313-10PC).  Do 
;	not use the 4 MHz version of the microcontroller (e.g., AT90S2313-4PC)--if 
;	you do, you will be exceeding the absolute maximum rating by more than a 
;	factor of 2.  Atmel has this to say in their datasheet:
;
;	NOTICE:	Stresses beyond those listed under "Absolute Maximum Ratings"
;			may cause permanent damage to the device. This is a stress
;			rating only and functional operation of the device at these or
;			other conditions beyond those indicated in the operational 
;			sections of this specification is not implied. Exposure to 
;			absolute maximum rating conditions for extended periods may 
;			affect device reliability. 
;
;	Data is transmitted over the RS-232 serial port at a baud rate of 115,200 
;	bps.  The frequency of the clock applied to the ISO 7816 CLK contact is 
;	assumed to be 4.608 MHz.  See the pin assignments below for other 
;	assumptions.
;-------------------------------------------------------------------------------

.include "2313def.inc"

;-------------------------------------------------------------------------------
; AVR clock frequency is 9.216 MHz --> period is 108.5 ns
; ISO clock frequency is 4.608 MHz --> period is 217.0 ns
; ISO baud rate is ISO clock / 372, or 12387 bps
; ISO bit width is 80.73 us
; ISO data is start bit plus 8 data bits plus (even) parity bit plus
; two stop bits, 12 bits total for 969 us
; ISO RST to start bit is 520 us
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; TIMER OVERFLOW TIMES for 9.216 MHz clock frequency
; 
; prescale  Timer0     Timer1
; --------  ---------  ---------
; CK         27.78 us    7.11 ms
; CK/8      222.20 us   56.90 ms
; CK/64       1.78 ms  455.10 ms
; CK/256      7.11 ms    1.82 s
; CK/1024    28.44 ms    7.30 s
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
; GLOBAL REGISTERS
;-------------------------------------------------------------------------------
.def	u_sr			= r1	; used to save SREG in ISRs
.def	u_buffer		= r14	; serial port data byte

.def	u_tmp			= r16	; scratch register
.def	u_bit_cnt		= r17	; number of bits transmitted or received for
								; current byte of data
.def	u0				= r18	; always all bits 0
.def	u_state			= r19	; bit-field state
.def	u_xmit_char		= r20	; byte to transmit to PC or card
.def	A				= r21	; scratch register
.def	B				= r22	; available for debug
.def	C				= r23	; available for debug
.def	u_t0_counter	= r24	; count of Timer0 overflows
.def	parity_bit		= r25	; parity bit for serial data

;-------------------------------------------------------------------------------
;	Bit positions in u_state
;-------------------------------------------------------------------------------
.equ	RD=0					; receiving data from card
.equ	TD=1					; transmitting data to card

;-------------------------------------------------------------------------------
;	Timer1 constants for bit spacings.
;-------------------------------------------------------------------------------
.equ	BS1H	= 0xFD			; 1.0 bit spacing
.equ	BS1L	= 0x18
.equ	BS1_5H	= 0xFB			; 1.5 bit spacings
.equ	BS1_5L	= 0xD4

;-------------------------------------------------------------------------------
;	Reset and Interrupt Vectors
;-------------------------------------------------------------------------------
.cseg

.org 	$0000
rjmp	RESET_HANDLER			; Reset handler
.org	ICP1addr
rjmp	INT_ICP					; Input Capture interrupt handler
.org	OVF1addr
rjmp	INT_OVF_TIMER1			; Timer1 overflow interrupt handler
.org	OVF0addr
rjmp	INT_OVF_TIMER0			; Timer0 overflow interrupt handler
.org 	URXCaddr
rjmp	INT_RX_CHAR 			; UART receive char interrupt handler

;-------------------------------------------------------------------------------
;	PIN ASSIGNMENTS
;-------------------------------------------------------------------------------
; ******
; PORT B
; ******
;
;   IN: SCK - clock for programming device and ISO RESET -- from RS-232 RTS
;   |   OUT: MISO - to RS-232 CTS
;   |   |   IN: MOSI - from RS-232 RTS inverted
;   |   |   |   IN: from ISO card slot switch; high means switch open and card in
;   |   |   |   |   OUT: OC1 - input to voltage follower for VCC
;   |   |   |   |   |   NOT USED
;   |   |   |   |   |   |   OUT: AIN1 - write LED - low on, high off 
;   |   |   |   |   |   |   |   OUT : AIN0 - VCC for LED -- always high
;   |   |   |   |   |   |   |   |
; +---+---+---+---+---+---+---+---+
; | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
; +---+---+---+---+---+---+---+---+
;  19  18  17  16  15  14  13  12
;
; ******
; PORT D
; ******
;
;   N/A
;   |   IN/OUT: ICP - I/O on ISO card slot
;   |   |   OUT: T1 - RESET on ISO card slot
;   |   |   |   OUT: T0 - 4053 MUX enable -- low to enable
;   |   |   |   |   OUT: INT1 - alternate clock enable
;   |   |   |   |   |   OUT: INT0 - ISO VCC Mux select -- high for VCC
;   |   |   |   |   |   |   OUT: TXD - RS-232 transmit data
;   |   |   |   |   |   |   |   IN : RXD - RS-232 receive data 
;   |   |   |   |   |   |   |   |
; +---+---+---+---+---+---+---+---+
; | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
; +---+---+---+---+---+---+---+---+
;      11   9   8   7   6   3   2
;-------------------------------------------------------------------------------

;-------------------------------
; Start of code on device reset.
;-------------------------------
RESET_HANDLER:

	;---------------------------------------------------------------------------
	;	Initialize the stack pointer.
	;---------------------------------------------------------------------------
	ldi		A,RAMEND		
	out		SPL,A

	;---------------------------------------------------------------------------
	;	u0 is initialized to all bits 0 and never changed.  It is used for 
	;	setting registers and output pins to 0.
	;---------------------------------------------------------------------------
	clr		u0

	;---------------------------------------------------------------------------
	;	Disable external interrupts.
	;---------------------------------------------------------------------------
	out		GIMSK,u0	

	;---------------------------------------------------------------------------
	;	Initialize ports B and D.
	;---------------------------------------------------------------------------
	ldi		A,0x50		; init state of PORT B
	out		PORTB,A
	ldi		A,0x76		; init state of PORT D
	out		PORTD,A		
	ldi		A,0x4F		; set PORT B directions
	out		DDRB,A
	ldi		A,0x7E		; set PORT D directions
	out		DDRD,A
	
	rcall	TURN_OFF_LEDS
	sbi		PORTB,6		; to RS-232 CTS
	sbi		PORTB,3		; VCC to OP-AMP voltage follower

	sbi		PORTD,5		; disable ISO reset
	cbi		PORTD,3		; use 4.608 MHz ISO clock 
	sbi		PORTD,2		; ISO VCC Mux
	cbi		PORTD,4		; enable 4053 MUX (active low)

	;---------------------------------------------------------------------------
	;	Initialize the UART (serial data interface to the PC).
	;---------------------------------------------------------------------------
	rcall	UART_INIT

	;---------------------------------------------------------------------------
	;	Enable interrupts.
	;---------------------------------------------------------------------------
	sei

	;---------------------------------------------------------------------------
	;	Wait for a card to be inserted.
	;---------------------------------------------------------------------------
wait_for_card_in:

	;---------------------------------------------------------------------------
	;	Read the state of the ISO card slot switch.  Card is inserted when 
	;	this pin is HIGH.
	;---------------------------------------------------------------------------
	sbis    PINB,4           ; skip if bit in I/O register set
	rjmp	wait_for_card_in

	;---------------------------------------------------------------------------
	;	Card is inserted.  Wait for start of RESET signal (RS-232 RTS line) 
	;	from serial port.
	;---------------------------------------------------------------------------
wait_for_reset_start:
	
	;---------------------------------------------------------------------------
	;	RESET starts when this pin is LOW
	;---------------------------------------------------------------------------
	sbic    PINB,7          ; skip if bit in I/O register cleared
	rjmp	wait_for_reset_start

	;---------------------------------------------------------------------------
	;	RESET is asserted.  Apply LOW to ISO RST contact to reset the card.
	;---------------------------------------------------------------------------
	cbi		PORTD,5

	;---------------------------------------------------------------------------
	;	Wait for RS-232 RTS line to indicate end of RESET.
	;---------------------------------------------------------------------------
wait_for_reset_end:

	;---------------------------------------------------------------------------
	;	RESET ends when this pin is HIGH
	;---------------------------------------------------------------------------
	sbis    PINB,7
	rjmp	wait_for_reset_end

	;---------------------------------------------------------------------------
	;	Apply HIGH to ISO RST contact to reset the card.
	;---------------------------------------------------------------------------
	sbi		PORTD,5
	
	cbi		PORTB,2			; initialize debug output on spare pin	

	;---------------------------------------------------------------------------
	;	Set up for the start bit (from first byte of ATR) from card.
	;---------------------------------------------------------------------------
	rcall	SETUP_FOR_START_BIT

	rjmp	wait_for_card_in

; end of AVR reset handler

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;	FUNCTIONS
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
TURN_ON_WRITE_LED:
	sbi		PORTB,0		; write LED VCC
	cbi		PORTB,1		; write LED ground
	ret	

TURN_ON_READ_LED:
	cbi		PORTB,0		; read LED ground
	sbi		PORTB,1		; read LED VCC
	ret	

TURN_OFF_LEDS:
	;---------------------------------------------------------------------------
	;	Applying VCC to both anode and cathode of LEDs turns them off.
	;---------------------------------------------------------------------------
	sbi		PORTB,0		; set to VCC
	sbi		PORTB,1		; set to VCC
	ret

UART_INIT:
	ldi		A,4			; 115200 baud for 9.216 MHz clock
	out		UBRR,A
	ldi		A,0x9C		; rcv & xmit & rcv interrupt & 9-bit chars
	out		UCR,A		; 9th bit is used for even parity
	ret

CLEAR_TIMER_INTERRUPTS:
	;---------------------------------------------------------------------------
	;	Clear timer/counter interrupts.  These are cleared by writing a 
	;	logical "1" to the flags in the Timer/Counter Interrupt FLAG Register 
	;	(TIFR).
	;---------------------------------------------------------------------------
	ldi		u_tmp,(1<<TOV1)+(1<<ICF1)+(1<<TOV0)	
	out		TIFR,u_tmp
	ret

;-------------------------------------------------------------------------------
;	This sets up for the input capture interrupt to fire on the leading edge 
;	of the start bit from the card.
;-------------------------------------------------------------------------------
SETUP_FOR_START_BIT:

	;---------------------------------------------------------------------------
	;	Disable timer interrupts.
	;---------------------------------------------------------------------------
	out		TIMSK,u0

	;---------------------------------------------------------------------------
	;	Set ISO I/O line to input.
	;---------------------------------------------------------------------------
	sbi		PORTD,6
	cbi     DDRD,6
	cbi		PORTD,6

	;---------------------------------------------------------------------------
	;	Set our state to read data.
	;---------------------------------------------------------------------------
	ldi		u_state,1<<RD

	;---------------------------------------------------------------------------
	;	Clear timer/counter interrupts.
	;---------------------------------------------------------------------------
	rcall	CLEAR_TIMER_INTERRUPTS

	;---------------------------------------------------------------------------
	;	Zero Timer0 counter.
	;---------------------------------------------------------------------------
	out		TCNT0,u0

	;---------------------------------------------------------------------------
	;	Zero Timer1 counter.
	;---------------------------------------------------------------------------
	out		TCNT1H,u0
	out		TCNT1L,u0

	;---------------------------------------------------------------------------
	;	Set clock prescale for Timer0.
	;---------------------------------------------------------------------------
	ldi		u_tmp,5				; CK/1024 -- at 9.216 MHXZ this gives 28 ms
	out		TCCR0,u_tmp
	ldi		u_t0_counter,12		; # of Timer0 interrupts before disabling Timer0.

	;---------------------------------------------------------------------------
	;	Set up the input capture to look for a falling slope, which will 
	;	represent the leading edge of the start bit.
	;---------------------------------------------------------------------------
	clr		u_bit_cnt			; clear bit counter
	out		TCCR1A,u0			; Timer/Counter1 disconnected from out pin OC1
	ldi		u_tmp,(1<<ICNC1)+(1<<CS10)
	out		TCCR1B,u_tmp		; Input capture noise canceller, capture at
								; falling edge, count at CK rate
	ldi		u_tmp,(1<<TOIE0)+(1<<TICIE)
	out		TIMSK,u_tmp			; interrupt at Timer0 and input capture

	ret

;-------------------------------------------------------------------------------
;	UART receive character interrupt vector.  This is called when a character 
;	is received from the PC over the serial port.
;-------------------------------------------------------------------------------
INT_RX_CHAR:
	in		u_sr,SREG			; save the status register

	in		u_buffer,UDR 		; read serial data

	;---------------------------------------------------------------------------
	;	Save the parity bit.
	;---------------------------------------------------------------------------
	ldi		parity_bit,1
	sbis	UCR,RXB8
	ldi		parity_bit,0

	;---------------------------------------------------------------------------
	;	Set ISO I/O line direction to output.  We will be transmitting the 
	;	received byte to the card.
	;---------------------------------------------------------------------------
	sbi		PORTD,6
	sbi     DDRD,6

	;---------------------------------------------------------------------------
	;	Clear ISO I/O line (this represents the leading edge of the start bit).
	;---------------------------------------------------------------------------
	cbi		PORTD,6

	;---------------------------------------------------------------------------
	;	Set up Timer1 control register.
	;---------------------------------------------------------------------------
	out		TCCR1A,u0			; Timer/Counter1 disconnected from out pin OC1
	ldi		u_tmp,1
	out		TCCR1B,u_tmp		; count at CK rate
	
	;---------------------------------------------------------------------------
	;	Clear timer/counter interrupts.
	;---------------------------------------------------------------------------
	rcall	CLEAR_TIMER_INTERRUPTS

	;---------------------------------------------------------------------------
	;	The only Timer interrupt we want now is Timer1 overflow (to transmit 
	;	data bits to the card).
	;---------------------------------------------------------------------------
	ldi		u_tmp,(1<<TOIE1)	; interrupt at TC01 
								; (turn off T0 and input capture)
	out		TIMSK,u_tmp			; set timer interrupt mask

	;---------------------------------------------------------------------------
	;	Set the current value of Timer1 so that it overflows 1 bit spacing 
	;	from now (which is when the start bit ends and the first data bit 
	;	begins).
	;---------------------------------------------------------------------------
	ldi		u_tmp,BS1H
	out		TCNT1H,u_tmp
	ldi		u_tmp,BS1L
	out		TCNT1L,u_tmp

	;---------------------------------------------------------------------------
	;	Set up our state.
	;---------------------------------------------------------------------------
	mov		u_xmit_char,u_buffer
	ldi		u_state,1<<TD		; set Transmit Data flag (clear all others)
	clr		u_bit_cnt

	;---------------------------------------------------------------------------
	;	Turn on the write LED.
	;---------------------------------------------------------------------------
	rcall	TURN_ON_WRITE_LED

	out		SREG,u_sr			; restore the status register
	reti						; return from ISR

;-------------------------------------------------------------------------------
;	Input capture interrupt vector.  This is called at the leading edge of the 
;	start bit (data sent from card).
;-------------------------------------------------------------------------------
INT_ICP:
	in		u_sr,SREG			; save the status register

	cli							; disable interrupts

	clr		u_buffer			; clear buffer
	clr		u_bit_cnt			; clear bit counter
	out		TCCR0,u0			; disable Timer0 counter
	out		TCNT1H,u0
	out		TCNT1L,u0

	;---------------------------------------------------------------------------
	;	Set up Timer1 to go off 1.5 bit spacings from now.  This gets us to 
	;	the middle of the first data bit (1 bit spacing for start bit plus 1/2 
	;	bit spacing for data bit).
	;---------------------------------------------------------------------------
	out		TCCR1A,u0			; Timer/Counter1 disconnected from out pin OC1
	ldi		u_tmp,1
	out		TCCR1B,u_tmp		; count at CK rate
	rcall	CLEAR_TIMER_INTERRUPTS
	ldi		u_tmp,(1<<TOIE1)
	out		TIMSK,u_tmp			; interrupt at TC01 
								; (turn off T0 and input capture)

	ldi		u_tmp,BS1_5H
	out		TCNT1H,u_tmp
	ldi		u_tmp,BS1_5L
	out		TCNT1L,u_tmp

	sei							; enable interrupts
	out		SREG,u_sr			; restore the status register

	rcall	TURN_ON_READ_LED	; turn on read LED

	reti						; return from ISR


;-------------------------------------------------------------------------------
;	Timer0 overflow interrupt vector.  Timer0 is used to turn off the read LED 
;	and clear all timer interrupts.  Because Timer0 is only an 8-bit value, 
;	its overflow period is too short for the duration needed to insure that 
;	all data transfers have completed.  Therefore, we use a counter 
;	(u_t0_counter) that gets decremented by 1 for each Timer0 overflow.  When 
;	the counter is 0, the entire wait period ends.
;-------------------------------------------------------------------------------
INT_OVF_TIMER0:
	in		u_sr,SREG			; save the status register

	dec		u_t0_counter
	breq	int_ovf_timer0_clear
	out		TCNT0,u0			; zero Timer0 counter
	rjmp	int_ovf_timer0_end

int_ovf_timer0_clear:
	;---------------------------------------------------------------------------
	;	Turn off LEDs.
	;---------------------------------------------------------------------------
	rcall	TURN_OFF_LEDS

	;---------------------------------------------------------------------------
	;	Stop Timer0.
	;---------------------------------------------------------------------------
	out		TCCR0,u0

	;---------------------------------------------------------------------------
	;	Disable all timer interrupts.
	;---------------------------------------------------------------------------
	out		TIMSK,u0
	;---------------------------------------------------------------------------
	;	Clear timer/counter interrupts.
	;---------------------------------------------------------------------------
	rcall	CLEAR_TIMER_INTERRUPTS

int_ovf_timer0_end:
	out		SREG,u_sr			; restore the status register
	reti						; return from ISR

;-------------------------------------------------------------------------------
;	Timer1 overflow interrupt vector.  Timer1 is used to read and write data 
;	on the ISO I/O line.  A single bit of data is read (data sent from card) 
;	or written (data sent to card) by this ISR.
;-------------------------------------------------------------------------------
INT_OVF_TIMER1:
	in		u_sr,SREG			; save the status register

	;---------------------------------------------------------------------------
	;	Set up Timer1 to overflow 1 bit spacing later.
	;---------------------------------------------------------------------------
	ldi		u_tmp,BS1H
	out		TCNT1H,u_tmp
	ldi		u_tmp,BS1L
	out		TCNT1L,u_tmp

	;---------------------------------------------------------------------------
	;	Toggle the state of pin 14.  This allows for bit transmit and sample 
	;	times to be monitored with an oscilloscope.
	;---------------------------------------------------------------------------
	rcall	DEBUG_TOGGLE_PIN14

	;---------------------------------------------------------------------------
	;	Update oue state info.
	;---------------------------------------------------------------------------
	inc		u_bit_cnt			; increment bit counter
	sbrs	u_state,TD			; skip next instruction if transmit bit set
	rjmp	tim1_receive		; goto receive

;-------------------------------------------------------------------------------
;	Transmits (to the card) are done here.
;-------------------------------------------------------------------------------
tim1_xmit:

	;---------------------------------------------------------------------------
	;	Check if this is the parity or stop bit.
	;---------------------------------------------------------------------------
	cpi		u_bit_cnt,9			; if this is the 9th bit
	breq	tim1_xmit_parity	; transmit the parity bit
	cpi		u_bit_cnt,10		; if this is the 10th bit
	breq	tim1_xmit_stop		; transmit the stop bit

	;---------------------------------------------------------------------------
	;	Transmit the next data bit.
	;---------------------------------------------------------------------------
	sbrc	u_buffer,0			; skip the next instruction if the next data bit 
								; (bit 0 in u_buffer) is a 0
	sbi		PORTD,6				; the next data bit is a 1, so transmit a 1
	sbrs	u_buffer,0			; skip the next instruction if the next data bit 
								; (bit 0 in u_buffer) is a 1
	cbi		PORTD,6				; the next data bit is a 0, so transmit a 0
	lsr		u_buffer			; shift buffer right
	
	out		SREG,u_sr			; restore status register
	reti						; return from ISR

;-------------------------------------------------------------------------------
;	Transmit of the parity bit (to the card) is done here.
;-------------------------------------------------------------------------------
tim1_xmit_parity:
	sbrc	parity_bit,0		; skip the next instruction if the parity bit 
								; (bit 0 in parity_bit) is a 0
	sbi		PORTD,6				; the parity bit is a 1, so transmit a 1
	sbrs	parity_bit,0		; skip the next instruction if the parity bit 
								; (bit 0 in parity_bit) is a 1
	cbi		PORTD,6				; the parity bit is a 0, so transmit a 0

	rjmp	tim1_ret			; jump to return

;-------------------------------------------------------------------------------
;	Transmit of the stop bit (to the card) is done here.
;-------------------------------------------------------------------------------
tim1_xmit_stop:
	sbi		PORTD,6				; stop bit is transmitted as a 1
	rjmp	tim1_xmit_complete	; jump to xmit complete

;-------------------------------------------------------------------------------
;	Transmit of all bits to the card is complete when we get her.  Now echo 
;	the character back to the serial port (PC) and set up for data to be sent 
;	by the card.
;-------------------------------------------------------------------------------
tim1_xmit_complete:
	;---------------------------------------------------------------------------
	;	The parity bit is transmitted as the 9th data bit (TXB8) in the UCR 
	;	register.
	;---------------------------------------------------------------------------
	sbrc	parity_bit,0		; skip the next instruction if the parity bit 
								; (bit 0 in parity_bit) is a 0
	sbi		UCR,TXB8			; the parity bit is a 1, so set TXB8 to a 1 in 
								; the UCR register
	sbrs	parity_bit,0		; skip the next instruction if the parity bit 
								; (bit 0 in parity_bit) is a 1
	cbi		UCR,TXB8			; the parity bit is a 0, so set TXB8 to a 0 in 
								; the UCR register

	out		UDR,u_xmit_char 	; transmit character over serial port
	rcall	SETUP_FOR_START_BIT	; set up for start bit from card
	rjmp	tim1_ret			; 

tim1_rcv_complete:
	;---------------------------------------------------------------------------
	;	Stop Timer0 and setup for next start bit from card.
	;---------------------------------------------------------------------------
	rcall	SETUP_FOR_START_BIT
	rjmp	tim1_ret
	
tim1_ret:
	out		SREG,u_sr			; restore status register
	reti						; return from ISR

;-------------------------------------------------------------------------------
;	Sampling of data sent from the card is done here.
;-------------------------------------------------------------------------------
tim1_receive:

	;---------------------------------------------------------------------------
	;	The 10th bit is the stop bit (8 data bit plus parity bit are first 9).
	;---------------------------------------------------------------------------
	cpi		u_bit_cnt,10
	breq	tim1_rcv_complete

	;---------------------------------------------------------------------------
	;	Sample the data bit from the card.  We use the carry bit to store that 
	;	bit value, then shift the carry bit into the data byte we are building.
	;---------------------------------------------------------------------------
	sec							; set carry
	sbis	PIND,6				; skip if ISO I/O line is high.
	clc							; clear carry
	ror		u_buffer			; shift carry into data and bit 0 into carry

	in		u_tmp,SREG			; save SREG for the carry bit

	;---------------------------------------------------------------------------
	;	Exit if this is not the parity bit.
	;---------------------------------------------------------------------------
	cpi		u_bit_cnt,9			; must sample parity bit
	brne	tim1_ret			; exit interrupt

	;---------------------------------------------------------------------------
	;	Save the parity bit, which is in bit 7 that was just read.
	;---------------------------------------------------------------------------
	ldi		parity_bit,1
	sbrs	u_buffer,7
	ldi		parity_bit,0

	out		SREG,u_tmp			; restore SREG that has the carry bit 
								; (bit 0 of data from card)

	;---------------------------------------------------------------------------
	;	At this point, the parity bit is in bit 7 of u_buffer, bits 7 through 
	;	1 of the data from the card are in bits 6 through 0 of u_buffer, and 
	;	bit 0 of the data from the card is in the carry bit.  The "rol" 
	;	instruction is "Rotate Left Through Carry", which will shift all bits 
	;	in u_buffer left by one and put the carry bit in bit 0 of u_buffer.  
	;	This gives us the data byte sent from to card.
	;---------------------------------------------------------------------------
	rol		u_buffer

	;---------------------------------------------------------------------------
	;	Now transmit the data byte received from the card to the serial port.
	;---------------------------------------------------------------------------
	mov		u_xmit_char,u_buffer

	sbrc	parity_bit,0		; if LSB in buffer is 1
	sbi		UCR,TXB8			; set transmit to 1
	sbrs	parity_bit,0		; if LSB in buffer is 0
	cbi		UCR,TXB8			; set transmit to 0

	out		UDR,u_xmit_char 	; send character to PC

	rjmp	tim1_ret			; exit ISR

;-------------------------------------------------------------------------------
;	This toggles the state of pin 14.
;-------------------------------------------------------------------------------
DEBUG_TOGGLE_PIN14:
	in		u_tmp,PORTB	; read the state of PORT B
	sbrc	u_tmp,2		; skip if bit is cleared
	cbi		PORTB,2		; bit is set, so clear it
	sbrs	u_tmp,2		; skip if bit is set
	sbi		PORTB,2		; bit is clear, so set it
	ret

;-------------------------------------------------------------------------------
;	EEPROM data.
;-------------------------------------------------------------------------------
.eseg
.org 0
EE_00:
.db 00,00
.db 0x5A, 0
.db 0x98, 0
